import datetime
from dataclasses import dataclass
from pathlib import Path
from typing import Iterator, Tuple

from imageio_ffmpeg import count_frames_and_secs, read_frames

from vhcalc.tools.imghash import FRAME_SIZE


@dataclass
class MetaData:
    fps: float
    duration: float
    nb_frames: int


def build_reader_frames(
    media: Path,
    nb_seconds_to_extract: float = 0,
    seek_to_middle: bool = False,
    ffmpeg_reduce_verbosity: bool = True,
) -> Tuple[Iterator[bytes], MetaData]:
    meta = {}
    ffmpeg_seek_input_cmd = []
    ffmpeg_seek_output_cmd = []

    if ffmpeg_reduce_verbosity or seek_to_middle:
        meta = next(read_frames(str(media)))

    # extract a (frame's) chunk around/in middle of the media
    # https://trac.ffmpeg.org/wiki/Seeking#Cuttingsmallsections
    if ffmpeg_reduce_verbosity:
        ffmpeg_seek_input_cmd += "-hide_banner -nostats -nostdin".split(" ")

    if seek_to_middle:
        ffmpeg_seek_input_cmd += (
            f"-ss {str(datetime.timedelta(seconds=meta['duration'] // 2))}".split(" ")
        )

    if nb_seconds_to_extract:
        ffmpeg_seek_output_cmd += [
            "-frames:v",
            str(round(nb_seconds_to_extract * meta["fps"])),
        ]
    # rescale output frame to 32x32
    video_filters = f"scale=width={FRAME_SIZE}:height={FRAME_SIZE}"

    reader = read_frames(
        str(media),
        input_params=[*ffmpeg_seek_input_cmd],
        output_params=[
            *ffmpeg_seek_output_cmd,
            *f"-vf {video_filters}".split(" "),
            *"-pix_fmt gray".split(" "),
        ],
        bits_per_pixel=8,
    )
    meta = next(reader)
    metadata = MetaData(
        fps=meta["fps"],
        duration=meta["duration"],
        nb_frames=count_frames_and_secs(media)[0],
    )

    return reader, metadata



from pathlib import Path

# https://pypi.org/project/click-pathlib/
from tempfile import gettempdir
from typing import Optional

from rich import get_console
from rich.console import Console

from vhcalc.tools.chunk import chunks
from vhcalc.tools.imghash import imghash_to_bytes, rawframe_to_imghash
from vhcalc.tools.progress_bar import configure_progress_bar

from .reader_frames import build_reader_frames


def export_imghash_from_media(
    input_media: Path,
    output_file: Optional[Path] = None,
    chunk_nb_seconds: int = 15,
    console: Optional[Console] = None,
) -> Path:
    console = console or get_console()
    # Read a video file
    it_reader_frame, media_metadata = build_reader_frames(input_media)
    console.print(media_metadata)
    nb_frames_to_read = media_metadata.nb_frames
    chunk_size = int(media_metadata.fps * chunk_nb_seconds)
    #
    console.print(f"Number of frames to read: {nb_frames_to_read}")
    console.print(f"Chunk size (nb frames): {chunk_size}")

    # manage export
    output_file = (
        Path(output_file)
        if output_file
        # https://bandit.readthedocs.io/en/latest/plugins/b108_hardcoded_tmp_directory.html
        else Path(gettempdir()) / f"{input_media.name}.{media_metadata.fps}fps.phash"
    )
    output_file.unlink(missing_ok=True)
    console.print(f"output_file: {str(output_file)}")

    # configure progress bar
    progress_bar, pb_task_id = configure_progress_bar(
        input_media.name, nb_frames_to_read * 8, console
    )
    pb_advance = chunk_size * 8

    # processing (export)
    with progress_bar:
        gen_imghashes = map(rawframe_to_imghash, it_reader_frame)
        gen_chunk_imghashes = chunks(gen_imghashes, chunk_size)
        for chunk_imghashes in gen_chunk_imghashes:
            with output_file.open("ab") as fo:
                for frame_hash_binary in map(imghash_to_bytes, chunk_imghashes):
                    fo.write(frame_hash_binary)
            progress_bar.update(pb_task_id, advance=pb_advance, refresh=True)
    return output_file



from datetime import timedelta
from typing import Optional, Tuple

from rich import get_console
from rich.console import Console
from rich.progress import (
    BarColumn,
    DownloadColumn,
    Progress,
    Task,
    TaskID,
    TextColumn,
    TimeRemainingColumn,
    TransferSpeedColumn,
)
from rich.text import Text


class TimeElapsedOverRemainingColumn(TimeRemainingColumn):
    """Renders remaining on elapsed time."""

    def render(self, task: Task) -> Text:
        """Show times elapsed/remaining."""
        elapsed_delta = timedelta(seconds=int(task.elapsed or -1))
        remaining = super().render(task)
        return Text(f"{str(elapsed_delta)}/{remaining}", style="progress.remaining")


def configure_progress_bar(
    filename: str, total: int, console: Optional[Console] = None
) -> Tuple[Progress, TaskID]:
    progress = Progress(
        TextColumn("[bold blue]{task.fields[filename]}", justify="right"),
        BarColumn(bar_width=None),
        "[progress.percentage]{task.percentage:>3.1f}%",
        "•",
        DownloadColumn(),
        "•",
        TransferSpeedColumn(),
        "•",
        TimeElapsedOverRemainingColumn(),
        console=console or get_console(),
    )
    task_id = progress.add_task(
        "build&export images hashes", filename=filename, start=True
    )
    progress.update(task_id, total=total)
    return progress, task_id



# -*- coding: utf-8 -*-
import pathlib
from importlib.metadata import version
from typing import Iterable, Optional

import click
from click_path import GlobPaths
from loguru import logger

import vhcalc.services as services


@click.version_option(version=version("vhcalc"), prog_name="vhcalc")
@click.group()
def cli() -> None:
    pass


@cli.command(
    short_help="extracting and exporting binary video hashes (fingerprints) from any video source"
)
@click.option(
    "--medias_pattern",
    "-r",
    required=True,
    type=GlobPaths(
        files_okay=True,
        dirs_okay=False,
        readable_only=True,
        at_least_one=True,
    ),
    help="Pattern to find medias",
)
@click.option(
    "--output-file",
    "-o",
    default=None,
    type=click.Path(writable=True, path_type=pathlib.Path),
    help="File where to write images hashes.",
)
@logger.catch
def export_imghash_from_media(
    medias_pattern: Iterable[pathlib.Path], output_file: Optional[pathlib.Path]
) -> None:
    """This script exporting binary images hashes (fingerprints)
    from (any) media (video file)
    \f"""
    for media in medias_pattern:
        services.export_imghash_from_media(media, output_file)


if __name__ == "__main__":
    cli()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vhcalc', 'vhcalc.services', 'vhcalc.tools']

package_data = \
{'': ['*']}

install_requires = \
['Distance>=0.1.3,<0.2.0',
 'ImageHash>=4.2.1,<5.0.0',
 'bitstring>=3.1.9,<4.0.0',
 'click-path>=0.0.5,<0.0.6',
 'click-pathlib>=2020.3.13,<2021.0.0',
 'click==8.0.4',
 'colorama==0.4.4',
 'imageio-ffmpeg>=0.4.5,<0.5.0',
 'loguru>=0.6.0,<0.7.0',
 'numpy>=1.22.3,<2.0.0',
 'rich[spinner]>=11.2.0,<12.0.0']

entry_points = \
{'console_scripts': ['export_imghash_from_media = '
                     'vhcalc.app:export_imghash_from_media']}

setup_kwargs = {
    'name': 'vhcalc',
    'version': '0.2.5',
    'description': "It's a client-side library that implements a custom algorithm for extracting video hashes (fingerprints) from any video source.",
    'long_description': "[![PRs Welcome](https://img.shields.io/badge/PRs-welcome-brightgreen.svg?style=flat-square)](http://makeapullrequest.com)\n[![Conventional Commits](https://img.shields.io/badge/Conventional%20Commits-1.0.0-yellow.svg?style=flat-square)](https://conventionalcommits.org)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![Github Actions](https://github.com/yoyonel/vhcalc/actions/workflows/python-check.yaml/badge.svg)](https://github.com/yoyonel/vhcalc/wayback-machine-saver/actions/workflows/python-check.yaml)\n\n[![PyPI Package latest release](https://img.shields.io/pypi/v/vhcalc.svg?style=flat-square)](https://pypi.org/project/vhcalc/)\n[![PyPI Package download count (per month)](https://img.shields.io/pypi/dm/vhcalc?style=flat-square)](https://pypi.org/project/vhcalc/)\n[![Supported versions](https://img.shields.io/pypi/pyversions/vhcalc.svg?style=flat-square)](https://pypi.org/project/vhcalc/)\n\n[![Docker Image Size (latest semver)](https://img.shields.io/docker/image-size/yoyonel/vhcalc?sort=semver)](https://hub.docker.com/r/yoyonel/vhcalc/)\n\n[![Coverage Status](https://coveralls.io/repos/github/yoyonel/vhcalc/badge.svg?branch=main)](https://coveralls.io/github/yoyonel/vhcalc?branch=main)\n\n# vhcalc\n\nIt's a client-side library that implements a custom algorithm for extracting video hashes (fingerprints) from any video source.\n\n## Getting Started\n\n### Prerequisites\n* [Python](https://www.python.org/downloads/)\n\n## Usage\n\n```sh\n$ export_imghash_from_media --help  \nUsage: export_imghash_from_media [OPTIONS]\n\n  This script exporting binary images hashes (fingerprints) from (any) media\n  (video file)\n\nOptions:\n  --version                       Show the version and exit.\n  -r, --medias_pattern PATH-OR-GLOB\n                                  Pattern to find medias  [required]\n  -o, --output-file PATH          File where to write images hashes.\n  --help                          Show this message and exit.\n```\n\n\n## Docker\n\nDocker hub: [yoyonel/vhcalc](https://hub.docker.com/r/yoyonel/vhcalc/)\n\n```sh\n$ docker run -it yoyonel/vhcalc:main --help\nUsage: export_imghash_from_media [OPTIONS]\n\n  This script exporting binary images hashes (fingerprints) from (any) media\n  (video file)\n\nOptions:\n  --version                       Show the version and exit.\n  -r, --medias_pattern PATH-OR-GLOB\n                                  Pattern to find medias  [required]\n  -o, --output-file PATH          File where to write images hashes.\n  --help                          Show this message and exit.\n```\n\n## Contributing\nSee [Contributing](contributing.md)\n\n## Authors\nLionel Atty <yoyonel@hotmail.com>\n\n\nCreated from [Lee-W/cookiecutter-python-template](https://github.com/Lee-W/cookiecutter-python-template/tree/1.1.2) version 1.1.2\n",
    'author': 'Lionel Atty',
    'author_email': 'yoyonel@hotmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/yoyonel/vhcalc',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"""
https://stackoverflow.com/questions/8290397/how-to-split-an-iterable-in-constant-size-chunks
"""
from itertools import chain, islice
from typing import Any, Iterator


def chunks(iterable: Iterator[Any], size: int) -> Iterator[Any]:
    """
    >>> list(chunks(range(10), 3))
    [[0, 1, 2], [3, 4, 5], [6, 7, 8], [9]]
    """
    iterator = iter(iterable)
    for first in iterator:
        yield list(chain([first], islice(iterator, size - 1)))
